#ifndef TABLEHELPER_H
#define TABLEHELPER_H

#include <QString>
#include <QPair>

class TableHelper
{
public:
    TableHelper();

    static QString createTable(const QString &tablename, const QList<QPair<QString,QString>> &fieldAndTypes);

    static QString createComment(const QString &tablename, const QList<QPair<QString,QString>> &fieldAndComments);

    static QString createPrimaryAlter(const QString &tablename);

    static QString createSeq(const QString &tablename);

    static QString createTrigger(const QString &tablename);

    static QString createDescription(const QString &entityname, const QList<QPair<QString,QString>> &fieldAndComments);
};

#endif // TABLEHELPER_H
